#!/usr/bin/perl

use strict;
# Disable warnings if NO DBI is available!
use warnings;

# To avoid File::basedir
my $lMyName = $0;
$lMyName =~ s/^.*\///g;
my $lMyNameBase = $lMyName;
$lMyNameBase =~ s/(\..*)?$//g;

my $basedir = '/home/zabbix/mysql_performance_monitor';
use lib "/home/zabbix/mysql_performance_monitor/lib";

use FromDualMySQLagent;

use constant INFO => 1;
use constant WARN => 2;
use constant ERR  => 3;
use constant DBG  => 4;

#basename
my $file = $basedir . '/etc/' . $lMyNameBase . '.conf';
my $gDomain = 'FromDual.MySQL';
my @configuration = &FromDualMySQLagent::readConfigurationFile($file);
my @sections = &FromDualMySQLagent::getSections(@configuration);
my %defaults = &FromDualMySQLagent::getDefaultParameter(\@configuration, 'default');

if ( $defaults{'Debug'} ) {
  &FromDualMySQLagent::log($defaults{'LogFile'}, INFO, 'Sections found: ' . join(', ', @sections));
}
if ( $defaults{'Debug'} ) {
  &FromDualMySQLagent::log($defaults{'LogFile'}, INFO, "Reading default section.");
}
if ( $defaults{'Debug'} == WARN ) {

  foreach my $p ( keys %defaults) {
    &FromDualMySQLagent::log($defaults{'LogFile'}, DBG, "$p - $defaults{$p}");
  }
}

foreach my $section ( @sections ) {

  if ( $section ne 'default' ) {

    my %parameter = ();
    %parameter = &FromDualMySQLagent::getParameter(\@configuration, $section);

    if ( $parameter{'Debug'} ) {
      &FromDualMySQLagent::log($parameter{'LogFile'}, INFO, "Processing section $section now...");
    }
    if ( $parameter{'Debug'} == WARN ) {
      &FromDualMySQLagent::log($parameter{'LogFile'}, INFO, "Parameters for section: $section");
      foreach my $p ( keys %parameter ) {
        &FromDualMySQLagent::log($parameter{'LogFile'}, DBG, "$p - $parameter{$p}");
      }
    }

    if ( $parameter{'Disabled'} eq 'true' ) {
      if ( $parameter{'Debug'} ) {
        &FromDualMySQLagent::log($parameter{'LogFile'}, INFO, "Skipping section: $section because it is disabled.");
      }
      next;
    }

    if ( $parameter{'Debug'} ) {
      &FromDualMySQLagent::log($parameter{'LogFile'}, INFO, "Modules for section $section: " . $parameter{'Modules'});
    }
    my @modules = split(/\s+/, $parameter{'Modules'});
    foreach my $module ( @modules ) {

      $parameter{'Domain'} = 'FromDual.MySQL.' . $module;

      &FromDualMySQLagent::log($parameter{'LogFile'}, INFO, "Processing module $module for section $section now...");

      if ( $module eq 'process' ) {
        eval {
          require FromDualMySQLprocess;
        };

        if ( $@ ) {
          &FromDualMySQLagent::log($parameter{'LogFile'}, ERR, "Module FromDualMySQLprocess does not exist.");
          foreach my $key ( $@ ) {
            &FromDualMySQLagent::log($parameter{'LogFile'}, DBG, $key);
          }
        }
        else {
          &FromDualMySQLprocess::processProcessInformation(\%parameter);
        }
      }
      elsif ( $module eq 'ndb' ) {
        eval {
          require FromDualMySQLndb;
        };

        if ( $@ ) {
          &FromDualMySQLagent::log($parameter{'LogFile'}, ERR, "Module FromDualMySQLndb does not exist.");
          foreach my $key ( $@ ) {
            &FromDualMySQLagent::log($parameter{'LogFile'}, DBG, $key);
          }
        }
        else {
          &FromDualMySQLndb::processNdbInformation(\%parameter);
        }
      }
      elsif ( $module eq 'master' ) {
        eval {
          require FromDualMySQLmaster;
        };

        if ( $@ ) {
          &FromDualMySQLagent::log($parameter{'LogFile'}, ERR, "Module FromDualMySQLmaster does not exist.");
          foreach my $key ( $@ ) {
            &FromDualMySQLagent::log($parameter{'LogFile'}, DBG, $key);
          }
        }
        else {
          &FromDualMySQLmaster::processMasterInformation(\%parameter);
        }
      }
      elsif ( $module eq 'slave' ) {
        eval {
          require FromDualMySQLslave;
        };

        if ( $@ ) {
          &FromDualMySQLagent::log($parameter{'LogFile'}, ERR, "Module FromDualMySQLslave does not exist.");
          foreach my $key ( $@ ) {
            &FromDualMySQLagent::log($parameter{'LogFile'}, DBG, $key);
          }
        }
        else {
          &FromDualMySQLslave::processSlaveInformation(\%parameter);
        }
      }
      elsif ( $module eq 'mysql' ) {
        eval {
          require FromDualMySQLmysql;
        };

        if ( $@ ) {
          &FromDualMySQLagent::log($parameter{'LogFile'}, ERR, "Module FromDualMySQLmysql does not exist.");
          foreach my $key ( $@ ) {
            &FromDualMySQLagent::log($parameter{'LogFile'}, DBG, $key);
          }
        }
        else {
          &FromDualMySQLmysql::processMysqlInformation(\%parameter);
        }
      }
      elsif ( $module eq 'myisam' ) {
        eval {
          require FromDualMySQLmyisam;
        };

        if ( $@ ) {
          &FromDualMySQLagent::log($parameter{'LogFile'}, ERR, "Module FromDualMySQLmyisam does not exist.");
          foreach my $key ( $@ ) {
            &FromDualMySQLagent::log($parameter{'LogFile'}, DBG, $key);
          }
        }
        else {
          &FromDualMySQLmyisam::processMyisamInformation(\%parameter);
        }
      }
      elsif ( $module eq 'innodb' ) {
        eval {
          require FromDualMySQLinnodb;
        };

        if ( $@ ) {
          &FromDualMySQLagent::log($parameter{'LogFile'}, ERR, "Module FromDualMySQLpbxt does not exist.");
          foreach my $key ( $@ ) {
            &FromDualMySQLagent::log($parameter{'LogFile'}, DBG, $key);
          }
        }
        else {
          &FromDualMySQLinnodb::processInnodbInformation(\%parameter);
        }
      }
      elsif ( $module eq 'pbxt' ) {
        eval {
          require FromDualMySQLpbxt;
        };

        if ( $@ ) {
          &FromDualMySQLagent::log($parameter{'LogFile'}, ERR, "Module FromDualMySQLpbxt does not exist.");
          foreach my $key ( $@ ) {
            &FromDualMySQLagent::log($parameter{'LogFile'}, DBG, $key);
          }
        }
        else {
          &FromDualMySQLpbxt::processPbxtInformation(\%parameter);
        }
      }
      elsif ( $module eq 'xtradb' ) {
        eval {
          require FromDualMySQLxtradb;
        };

        if ( $@ ) {
          &FromDualMySQLagent::log($parameter{'LogFile'}, ERR, "Module FromDualMySQLxtradb does not exist.");
          foreach my $key ( $@ ) {
            &FromDualMySQLagent::log($parameter{'LogFile'}, DBG, $key);
          }
        }
        else {
          &FromDualMySQLxtradb::processXtradbInformation(\%parameter);
        }
      }
      elsif ( $module eq 'aria' ) {
        eval {
          require FromDualMySQLaria;
        };

        if ( $@ ) {
          &FromDualMySQLagent::log($parameter{'LogFile'}, ERR, "Module FromDualMySQLaria does not exist.");
          foreach my $key ( $@ ) {
            &FromDualMySQLagent::log($parameter{'LogFile'}, DBG, $key);
          }
        }
        else {
          &FromDualMySQLaria::processAriaInformation(\%parameter);
        }
      }
      else {
        &FromDualMySQLagent::log($parameter{'LogFile'}, ERR, "Module $module does not exist.");
      }
    }
  }
}

print "1\n";
exit 0;
