package FromDualMySQLslave;

use strict;
use warnings;
use Time::HiRes qw( gettimeofday );

use constant DBG  => 4;
use constant INFO => 1;
use constant WARN => 2;
use constant ERR  => 3;

sub getSlaveStatus
{
  my $conf_ref = shift;
  my $dbh = shift;
  my $status_ref = shift;

  #       Master_Log_File:
  #   Read_Master_Log_Pos: 4
  # 
  #        Relay_Log_File: master-relay-bin.000001
  #         Relay_Log_Pos: 4
  # 
  # Relay_Master_Log_File:
  #   Exec_Master_Log_Pos: 0
  # 
  #      Slave_IO_Running: No
  #     Slave_SQL_Running: No
  # 
  # Seconds_Behind_Master: NULL

  $$status_ref{'Read_Master_Log_Pos'}   = '0';
  $$status_ref{'Relay_Log_Pos'}         = '0';
  $$status_ref{'Exec_Master_Log_Pos'}   = '0';
  $$status_ref{'Slave_IO_Running'}      = '0';
  $$status_ref{'Slave_SQL_Running'}     = '0';
  $$status_ref{'Seconds_Behind_Master'} = '0';

  my $sql = 'SHOW SLAVE STATUS';
  if ( $$conf_ref{'FetchMethod'} eq 'DBI' )
  {

    my $sth = $dbh->prepare($sql);
    if ( $sth->execute() )
    {
      if ( my $ref = $sth->fetchrow_hashref() )
      {
        $$status_ref{'Read_Master_Log_Pos'}   = $ref->{'Read_Master_Log_Pos'};
        $$status_ref{'Relay_Log_Pos'}         = $ref->{'Relay_Log_Pos'};
        $$status_ref{'Exec_Master_Log_Pos'}   = $ref->{'Exec_Master_Log_Pos'};
        $$status_ref{'Slave_IO_Running'}      = $ref->{'Slave_IO_Running'};
        $$status_ref{'Slave_SQL_Running'}     = $ref->{'Slave_SQL_Running'};
        $$status_ref{'Seconds_Behind_Master'} = defined($ref->{'Seconds_Behind_Master'}) ? $ref->{'Seconds_Behind_Master'} : 9999;
      }
      $sth->finish();
    }
  }
  elsif ( $$conf_ref{'FetchMethod'} eq 'mysql' ) {

    $sql .= '\G';
    my $cmd = 'mysql --user=' . $$conf_ref{'Username'} . ' --host=' . $$conf_ref{'MysqlHost'} . ' --port=' . $$conf_ref{'MysqlPort'} . ' --password=' . $$conf_ref{'Password'} . " --execute='" . $sql . "'";
    if ( $$conf_ref{'Debug'} ) {
      &FromDualMySQLagent::log($$conf_ref{'LogFile'}, DBG, "$cmd");
    }

    my @stdout = `$cmd 2>&1`;

    if ( $? != 0 ) {
      &FromDualMySQLagent::log($$conf_ref{'LogFile'}, ERR, "$cmd failed.");
    }
    else {

      foreach ( @stdout ) {
        chomp $_;

        if ( $_ =~ m/^\s*(\w+):\s+(.*)$/ ) {

          if ( $1 eq 'Read_Master_Log_Pos' ) {
            $$status_ref{'Read_Master_Log_Pos'} = $2;
          }
          elsif ( $1 eq 'Relay_Log_Pos' ) {
            $$status_ref{'Relay_Log_Pos'} = $2;
          }
          elsif ( $1 eq 'Exec_Master_Log_Pos' ) {
            $$status_ref{'Exec_Master_Log_Pos'} = $2;
          }
          elsif ( $1 eq 'Slave_IO_Running' ) {
            $$status_ref{'Slave_IO_Running'} = $2;
          }
          elsif ( $1 eq 'Slave_SQL_Running' ) {
            $$status_ref{'Slave_SQL_Running'} = $2;
          }
          elsif ( $1 eq 'Seconds_Behind_Master' ) {
            $$status_ref{'Seconds_Behind_Master'} = $2 eq 'NULL' ? 9999 : $2;
          }
        }
      }
    }
  }

  $$status_ref{'Slave_IO_Running'} = $$status_ref{'Slave_IO_Running'} eq 'Yes' ? 1 : 0;
  $$status_ref{'Slave_SQL_Running'} = $$status_ref{'Slave_SQL_Running'} eq 'Yes' ? 1 : 0;

  if ( $$conf_ref{'Debug'} == WARN )
  {
    &FromDualMySQLagent::log($$conf_ref{'LogFile'}, INFO, 'Master Status:');
    foreach my $key ( keys %$status_ref )
    {
      &FromDualMySQLagent::log($$conf_ref{'LogFile'}, DBG, "key: $key, value: $$status_ref{$key}");
    }
  }
}

sub processSlaveInformation
{
  my $parameter_ref = shift;

  my %hSlaveStatus;

  my @aSlaveStatusToSend = (
  'Read_Master_Log_Pos', 'Relay_Log_Pos', 'Exec_Master_Log_Pos', 'Slave_IO_Running'
, 'Slave_SQL_Running', 'Seconds_Behind_Master'
  );

  &FromDualMySQLagent::initValues(\%hSlaveStatus, \@aSlaveStatusToSend);

  my $dbh = &FromDualMySQLagent::getDatabaseConnection(\%$parameter_ref);

  &getSlaveStatus(\%$parameter_ref, $dbh, \%hSlaveStatus);

  &FromDualMySQLagent::releaseDatabaseConnection(\%$parameter_ref, $dbh);

  # Do some post calculations

  &FromDualMySQLagent::sendData(\%$parameter_ref, \%hSlaveStatus, \@aSlaveStatusToSend, $$parameter_ref{'Domain'});
}

1;
__END__
